"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const result_1 = tslib_1.__importDefault(require("./result"));
const context_1 = require("./uitls/context");
function createMockFunction(targetFn) {
    const createRunner = (args = null) => (injectContext) => {
        const context = Object.assign({}, context_1.createBaseContext(), injectContext);
        const returnVal = targetFn.apply(context, args ? args : []);
        return new result_1.default(returnVal, context);
    };
    const mockFunc = (...args) => {
        return {
            run: createRunner(args),
            r: createRunner(args)
        };
    };
    mockFunc.run = mockFunc.r = createRunner();
    return mockFunc;
}
function createMock(computed, computedName) {
    const target = computed[computedName];
    if (typeof target === 'function') {
        return createMockFunction(target);
    }
    else if (typeof target === 'object') {
        const mockObject = {};
        if (target.get) {
            mockObject.get = createMockFunction(target.get);
        }
        if (target.set) {
            mockObject.set = createMockFunction(target.set);
        }
        return mockObject;
    }
    else {
        throw new Error('Unexpected computed property.');
    }
}
function computed(component) {
    var _a;
    if (typeof component !== 'object' && typeof component !== 'function') {
        throw new Error('Illegal component. component must be object or function.');
    }
    const computed = component.options
        ? component.options.computed // VueConstructor
        : (_a = component) === null || _a === void 0 ? void 0 : _a.computed; // Not VueConstructor
    if (!computed)
        throw new Error('Not exists method.');
    const mockComputed = {};
    Object.keys(computed).forEach((key) => {
        mockComputed[key] = createMock({ ...computed }, key);
    });
    return mockComputed;
}
exports.default = computed;
