"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const result_1 = tslib_1.__importDefault(require("./result"));
const context_1 = require("./uitls/context");
const LIFECYCLE_HOOKS = [
    'beforeCreate',
    'created',
    'beforeMount',
    'mounted',
    'beforeUpdate',
    'updated',
    'beforeDestroy',
    'destroyed'
];
function createMockFunction(funcs, funcName) {
    const mockFuncs = { ...funcs };
    const createRunner = (args = null) => {
        Object.keys(funcs).forEach((k) => {
            if (funcName !== k)
                mockFuncs[k] = jest.fn();
        });
        const targetFunc = funcs[funcName];
        if (targetFunc.constructor.name === 'AsyncFunction') {
            return async (injectContext) => {
                const context = Object.assign({}, context_1.createBaseContext(), mockFuncs, injectContext);
                const returnVal = await targetFunc.apply(context, args ? args : []);
                return new result_1.default(returnVal, context);
            };
        }
        return (injectContext) => {
            const context = Object.assign({}, context_1.createBaseContext(), mockFuncs, injectContext);
            const returnVal = targetFunc.apply(context, args ? args : []);
            return new result_1.default(returnVal, context);
        };
    };
    const targetMockFunc = (...args) => {
        return {
            run: createRunner(args),
            r: createRunner(args)
        };
    };
    targetMockFunc.run = targetMockFunc.r = createRunner();
    return targetMockFunc;
}
function hooks(component, additionalHooks = []) {
    var _a;
    if (typeof component !== 'object' && typeof component !== 'function') {
        throw new Error('Illegal component. component must be object or function.');
    }
    const targetHooks = LIFECYCLE_HOOKS.concat(additionalHooks);
    const hooks = component.options
        ? // VueConstructor
            Object.keys(component.options).reduce((hooks, key) => {
                if (targetHooks.includes(key)) {
                    return {
                        ...hooks,
                        [key]: typeof component.options[key] === 'object'
                            ? component.options[key][0]
                            : component.options[key]
                    };
                }
                return hooks;
            }, {})
        : // Not VueConstructor
            Object.keys(component).reduce((hooks, key) => {
                if (targetHooks.includes(key))
                    return { ...hooks, [key]: component[key] };
                return hooks;
            }, {});
    if (Object.keys(hooks).length === 0)
        throw new Error('Not exists hook.');
    const methods = component.options
        ? component.options.methods // VueConstructor
        : (_a = component) === null || _a === void 0 ? void 0 : _a.methods; // Not VueConstructor
    const mockHookes = {};
    Object.keys(hooks).forEach((key) => {
        mockHookes[key] = createMockFunction({ ...hooks, ...methods }, key);
    });
    return mockHookes;
}
exports.default = hooks;
