"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const result_1 = tslib_1.__importDefault(require("./result"));
const context_1 = require("./uitls/context");
function createMockFunction(methods, methodName) {
    const mockMethods = { ...methods };
    const createRunner = (args = null) => {
        Object.keys(methods).forEach((k) => {
            if (methodName !== k)
                mockMethods[k] = jest.fn();
        });
        const targetMethod = methods[methodName];
        if (targetMethod.constructor.name === 'AsyncFunction') {
            return async (injectContext) => {
                const context = Object.assign({}, context_1.createBaseContext(), mockMethods, injectContext);
                const returnVal = await targetMethod.apply(context, args ? args : []);
                return new result_1.default(returnVal, context);
            };
        }
        return (injectContext) => {
            const context = Object.assign({}, context_1.createBaseContext(), mockMethods, injectContext);
            const returnVal = targetMethod.apply(context, args ? args : []);
            return new result_1.default(returnVal, context);
        };
    };
    const mockFn = (...args) => {
        return {
            run: createRunner(args),
            r: createRunner(args)
        };
    };
    mockFn.run = mockFn.r = createRunner();
    return mockFn;
}
function methods(component) {
    var _a;
    if (typeof component !== 'object' && typeof component !== 'function') {
        throw new Error('Illegal component. component must be object or function.');
    }
    const methods = component.options
        ? component.options.methods // VueConstructor
        : (_a = component) === null || _a === void 0 ? void 0 : _a.methods; // Not VueConstructor
    if (!methods)
        throw new Error('Not exists method.');
    const mockMethods = {};
    Object.keys(methods).forEach((key) => {
        mockMethods[key] = createMockFunction({ ...methods }, key);
    });
    return mockMethods;
}
exports.default = methods;
